/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "ticTacToeOptions.h"

TicTacToeOptions::TicTacToeOptions(std::string fileName, std::vector<int> defaultColors): mFileName(fileName) {
	setDefaultParameters(defaultColors);
	
	if(fileExists (mFileName)) {
		readConfigurationFile();
	} 	
}

TicTacToeOptions::~TicTacToeOptions() {
}

int TicTacToeOptions::getColor(int numPlayer) {
	return mColors[numPlayer];
}

std::vector<int> TicTacToeOptions::getColors() {
	return mColors;
}

int TicTacToeOptions::getNbPlayers() {
	return mColors.size()-1;
}

int TicTacToeOptions::getGridSize() {
	return mGridSize;
}

int TicTacToeOptions::getVictoryCondition() {
	return mVictoryCondition;
}

bool TicTacToeOptions::isAutoSelectionEnabled() {
	return mAutoSelection;
}

bool TicTacToeOptions::areColoredLightsEnabled() {
	return mColoredLights;
}



void TicTacToeOptions::setDefaultParameters(std::vector<int> defaultColors) {
	mColors = defaultColors;
	mGridSize = 4;
	mVictoryCondition = -1;
	mAutoSelection = true;
	mColoredLights = false;
}
				
void TicTacToeOptions::readConfigurationFile() {
	std::string line;
	std::ifstream file(mFileName.c_str());

	std::string separators = " \n";	
	std::vector<std::string> words;
		
	while(std::getline(file, line)) {
		if(line[0]!='#') {
			words = split(line, separators);
			if((words.size() >=3)&&(words[0]=="set")) {
				if(words[1] == "gridSize") {
					std::istringstream iss(words[2]);
					int value;
					iss >> value; 
					mGridSize = value;
				} else if(words[1] == "victoryCondition") {
					std::istringstream iss(words[2]);
					int value;
					iss >> value; 
					mVictoryCondition = value;
				} else if(words[1] == "colors") {
					for(int i=0;i<words.size()-2;i++) {
						std::istringstream iss(words[2+i]);
						int value;
						iss >> value; 
						if(i<=2) {
							mColors[i] = value;
						} else {
							mColors.push_back(value);
						}
					}
				} else if(words[1] == "autoSelection") {
					std::istringstream iss(words[2]);
					bool value;
					iss >> value; 
					mAutoSelection = value;					
				} else if(words[1] == "coloredLights") {
					std::istringstream iss(words[2]);
					bool value;
					iss >> value; 
					mColoredLights = value;					
				}
			}
		}
	}
}

void TicTacToeOptions::writeConfigurationFile() {
	std::ofstream file(mFileName.c_str(), std::ios::trunc);
	
	if ( !file ){
		std::cerr << "Cannot create configuration file." << std::endl;
		exit(1);
	}
	
	std::vector<std::string> comments;
	
	file << "## Tic-Tac-Toe (or Score Four) options file. \n \n";
	
	comments.push_back("The size of the grid.");
	file << writeParamDefinition("gridSize", mGridSize, comments);
	comments.clear();
	
	comments.push_back("Colors of the spheres.");
	comments.push_back("The first number is the color of free spheres.");	
	comments.push_back("The other numbers are the colors of the players.");		
	file << writeVectorParamDefinition("colors", mColors, comments);
	comments.clear();	
	
	comments.push_back("The victory condition is the number of points that a player have to mark to win the game.");
	comments.push_back("A player has one point for each line of spheres.");
	comments.push_back("If victoryCondition is non positive, the game ends when every sphere belongs to one of the players.");
	file << writeParamDefinition("victoryCondition", mVictoryCondition, comments);
	comments.clear();

	comments.push_back("If autoSelection is enabled, the spheres (or tori) are selected when the mouse is above.");
	file << writeParamDefinition("autoSelection", mAutoSelection, comments);
	comments.clear();	

	comments.push_back("Add colored light sources: one source for each player, the color is the color of the player.");
	comments.push_back("Colored light are beautiful, but too many light sources can decrease performance.");	
	file << writeParamDefinition("coloredLights", mColoredLights, comments);
	comments.clear();	
	
	file.close();	
}






